"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateEntrypointSource = validateEntrypointSource;
exports.isMainFieldValid = isMainFieldValid;
exports.isModuleFieldValid = isModuleFieldValid;
exports.isUmdMainFieldValid = isUmdMainFieldValid;
exports.isBrowserFieldValid = isBrowserFieldValid;
exports.isReactNativeFieldValid = isReactNativeFieldValid;
exports.isUmdNameSpecified = isUmdNameSpecified;
exports.validateEntrypoint = validateEntrypoint;
exports.default = validate;

var _project = require("./project");

var _entrypoint = require("./entrypoint");

var _messages = require("./messages");

var _errors = require("./errors");

var _utils = require("./utils");

var logger = _interopRequireWildcard(require("./logger"));

var _fastDeepEqual = _interopRequireDefault(require("fast-deep-equal"));

var _validatePackage = require("./validate-package");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

// this doesn't offer to fix anything
// just does validation
// used in build and watch
function validateEntrypointSource(entrypoint) {
  try {
    if (!entrypoint.source.startsWith(entrypoint.package.directory)) {
      throw new _errors.FatalError(`entrypoint source files must be inside their respective package directory but this entrypoint has specified its source file as ${entrypoint.configSource}`, entrypoint);
    }
  } catch (e) {
    if (e.code === "MODULE_NOT_FOUND") {
      throw new _errors.FatalError(_messages.errors.noSource(entrypoint.configSource), entrypoint);
    }

    throw e;
  }
}

function isMainFieldValid(entrypoint) {
  return entrypoint.main === (0, _utils.getValidStringFieldContentForBuildType)("main", entrypoint.package.name);
}

function isModuleFieldValid(entrypoint) {
  return entrypoint.module === (0, _utils.getValidStringFieldContentForBuildType)("module", entrypoint.package.name);
}

function isUmdMainFieldValid(entrypoint) {
  return entrypoint.umdMain === (0, _utils.getValidStringFieldContentForBuildType)("umd:main", entrypoint.package.name);
}

function isBrowserFieldValid(entrypoint) {
  return (0, _fastDeepEqual.default)(entrypoint.browser, (0, _utils.getValidObjectFieldContentForBuildType)("browser", entrypoint.package.name, entrypoint.module !== null));
}

function isReactNativeFieldValid(entrypoint) {
  return (0, _fastDeepEqual.default)(entrypoint.reactNative, (0, _utils.getValidObjectFieldContentForBuildType)("react-native", entrypoint.package.name, entrypoint.module !== null));
}

function isUmdNameSpecified(entrypoint) {
  return typeof entrypoint._config.umdName === "string";
}

function validateEntrypoint(entrypoint, log) {
  validateEntrypointSource(entrypoint);

  if (log) {
    logger.info(_messages.infos.validEntrypoint, entrypoint);
  }

  if (!isMainFieldValid(entrypoint)) {
    throw new _errors.FixableError(_messages.errors.invalidMainField, entrypoint);
  }

  if (log) {
    logger.info(_messages.infos.validMainField, entrypoint);
  }

  if (entrypoint.module !== null) {
    if (isModuleFieldValid(entrypoint)) {
      if (log) {
        logger.info(_messages.infos.validModuleField, entrypoint);
      }
    } else {
      throw new _errors.FixableError(_messages.errors.invalidModuleField, entrypoint);
    }
  }

  if (entrypoint.umdMain !== null) {
    if (isUmdMainFieldValid(entrypoint)) {
      if (isUmdNameSpecified(entrypoint)) {
        if (log) {
          logger.info(_messages.infos.validUmdMainField, entrypoint);
        }
      } else {
        throw new _errors.FixableError(_messages.errors.umdNameNotSpecified, entrypoint);
      }
    } else {
      throw new _errors.FixableError(_messages.errors.invalidUmdMainField, entrypoint);
    }
  }

  if (entrypoint.browser !== null) {
    if (typeof entrypoint.browser === "string" || !isBrowserFieldValid(entrypoint)) {
      throw new _errors.FixableError(_messages.errors.invalidBrowserField, entrypoint);
    } else if (log) {
      logger.info(_messages.infos.validBrowserField, entrypoint);
    }
  }

  if (entrypoint.reactNative !== null) {
    if (typeof entrypoint.reactNative === "string" || !isReactNativeFieldValid(entrypoint)) {
      throw new _errors.FixableError(_messages.errors.invalidReactNativeField, entrypoint);
    } else if (log) {
      logger.info(_messages.infos.validReactNativeField, entrypoint);
    }
  }
}

async function validate(directory) {
  let project = await _project.Project.create(directory);

  for (let pkg of project.packages) {
    (0, _validatePackage.validatePackage)(pkg);

    for (let entrypoint of pkg.entrypoints) {
      validateEntrypoint(entrypoint, true);
    }

    logger.info(_messages.infos.validPackageEntrypoints, pkg);
  }

  logger.success(_messages.successes.validProject);
}